@@ -28,6 +28,7 @@ typedef struct {
     int rel;
     int just;
     bool clip;
+    string *color;
 } strmod;
 
 class linedesc  {
@@ -47,7 +48,7 @@ public:
 	    if ( fc ) fillcolor = new string(*fc);
     }
 
-    linedesc(linedesc *l) {
+    linedesc(const linedesc *l) {
 	if ( l) {
 	    ld = l->ld;
 	    param = l->param;
@@ -111,7 +112,7 @@ public:
 
     shiftdesc(sides s=top_side, double p=0) : dir(s), param(p) { }
     
-    shiftdesc(shiftdesc *sh ) {
+    shiftdesc(const shiftdesc *sh ) {
 	if ( sh ) {
 	    dir = sh->dir;
 	    param = sh->param;
@@ -162,16 +163,23 @@ public:
     double size;	// Fontsize
     int relsz;		// True if the fontsize is relative
     bool clip;		// True if the string can only appear in the frame
+    string *color;	// color of the string
 
-    DisplayString() : string(), j(none), size(0), relsz(0), clip(true) {}
-    DisplayString(char *s, int ju=0, double sz=0, int rsz=0, bool c=true) :
-	string(s), j(ju), size(sz), relsz(rsz), clip(c) { }
-    DisplayString(string s, int ju=0, double sz=0, int rsz=0, bool c=true) :
-	string(s), j(ju), size(sz), relsz(rsz), clip(c) { }
-    DisplayString(DisplayString& ds) :
-	string(ds), j(ds.j), size(ds.size), relsz(ds.relsz), clip(ds.clip) { }
+    DisplayString() : string(), j(none), size(0), relsz(0), clip(true), 
+	color(0) { }
+    DisplayString(char *s, int ju=0, double sz=0, int rsz=0, bool c=true,
+	    string *col=0) :
+	string(s), j(ju), size(sz), relsz(rsz), clip(c), color(col) { }
+    DisplayString(string s, int ju=0, double sz=0, int rsz=0, bool c=true,
+	    string *col=0) :
+	string(s), j(ju), size(sz), relsz(rsz), clip(c), color(col) { }
+    DisplayString(const DisplayString& ds) :
+	string(ds), j(ds.j), size(ds.size), relsz(ds.relsz), clip(ds.clip),
+	color(ds.color) { 
+	    if ( color ) color = new string(*color);
+    }
     DisplayString(double e, const DisplayString *fmt=0) :
-	j(0), size(0), relsz(0), clip(true) {
+	j(0), size(0), relsz(0), clip(true), color(0) {
 	char *c = new char[64];
 	bool delf = false;
 
@@ -185,9 +193,16 @@ public:
 	delete[] c;
 	if ( delf ) delete fmt;
 	else { 
-	    j = fmt->j; size = fmt->size; relsz = fmt->relsz; clip = fmt->clip;
+	    j = fmt->j;
+	    size = fmt->size;
+	    relsz = fmt->relsz;
+	    clip = fmt->clip;
+	    if ( fmt->color ) color = new string(*fmt->color);
+	    else color = 0;
 	}
     }
+
+    ~DisplayString() { delete color; } 
 };
 
 // A grap coordinate system
@@ -249,7 +264,7 @@ public:
     coord *c;		// The coordinate scale that the tick is in
     
     tick() : where(0), size(0), side(top_side), prt(0), shift(), c(0) { }
-    tick(tick& t) :
+    tick(const tick& t) :
 	where(t.where), size(t.size), side(t.side), shift(), c(t.c) {
 	shiftcpy sc(&shift);
 	
@@ -321,7 +336,7 @@ public:
     }
 
     // To allow ticks and grids to share parse rules
-    grid(tick *t) : where(t->where),  desc(dotted,0,0), side(t->side),
+    grid(const tick *t) : where(t->where),  desc(dotted,0,0), side(t->side),
 	    prt(0), shift(), c(t->c) {
 	shiftcpy sc(&shift);
 
@@ -329,7 +344,7 @@ public:
 	for_each(t->shift.begin(), t->shift.end(), sc);
     }
 
-    grid(grid& g) : where(g.where), desc(g.desc), side(g.side), prt(0),
+    grid(const grid& g) : where(g.where), desc(g.desc), side(g.side), prt(0),
 		shift(), c(g.c) {
 	shiftcpy sc(&shift);
 
@@ -377,7 +392,7 @@ public:
     point(double xx, double yy, coord* cc) : x(xx), y(yy), c(cc) {
 	if ( c ) c->newpt(x, y);
     }
-    point(point *p) : x(p->x), y(p->y), c(p->c) {
+    point(const point *p) : x(p->x), y(p->y), c(p->c) {
 	if ( c ) c->newpt(x, y);
     }
     point(const point& p) : x(p.x), y(p.x), c(p.c) { }
@@ -474,7 +489,7 @@ public:
 		    arrow(false) { } ;
     linesegment(double xx, double yy, coord* cc, line *ll, DisplayString *s=0,
 	      linedesc *l=0, bool a=0);
-    linesegment(linesegment& ls) :
+    linesegment(const linesegment& ls) :
 	to(ls.to), desc(ls.desc), plotstr(0) , arrow(ls.arrow) {
 	if ( ls.from ) from = new point(ls.from);
 	if ( ls.plotstr ) plotstr = new DisplayString(*ls.plotstr);
@@ -500,7 +515,7 @@ public:
 	if (s) plotstr = new DisplayString(*s);
     }
     
-    line(line& l) : plotstr(0), desc(l.desc),  lastpoint(0) {
+    line(const line& l) : plotstr(0), desc(l.desc),  lastpoint(0) {
 	if ( l.plotstr ) {
 	    plotstr = new DisplayString(*l.plotstr);
 	}
@@ -533,7 +548,7 @@ public:
     
     plot(stringlist *s = 0, point *p =0, bool clip=true) : strs(s), loc(p) { }
     // copy constructors have to copy ...
-    plot( const plot& p ) : strs(0), loc(0) {
+    plot(const plot& p ) : strs(0), loc(0) {
 	if (p.loc) loc = new point(p.loc);
 
 	if ( p.strs ) {
@@ -576,7 +591,7 @@ public:
     circle(point *p, double r, linedesc *l=0) : center(p), rad(r), ld(solid) {
 	if ( l ) ld = *l;
     }
-    circle(circle& c) : center(&c.center), rad(c.rad), ld(c.ld) { }
+    circle(const circle& c) : center(&c.center), rad(c.rad), ld(c.ld) { }
 };
 
 class box {
